@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 2012-2018 Matt Martz
+# Copyright 2012 Matt Martz
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -36,7 +36,7 @@ except ImportError:
     gzip = None
     GZIP_BASE = object
 
-__version__ = '2.0.2'
+__version__ = '2.1.0'
 
 
 class FakeShutdownEvent(object):
@@ -435,14 +435,18 @@ if HTTPSConnection:
 
             SpeedtestHTTPConnection.connect(self)
 
-            kwargs = {}
             if ssl:
-                if hasattr(ssl, 'SSLContext'):
-                    kwargs['server_hostname'] = self.host
                 try:
+                    kwargs = {}
+                    if hasattr(ssl, 'SSLContext'):
+                        kwargs['server_hostname'] = self.host
                     self.sock = self._context.wrap_socket(self.sock, **kwargs)
                 except AttributeError:
-                    self.sock = ssl.wrap_socket(self.sock, **kwargs)
+                    self.sock = ssl.wrap_socket(self.sock)
+                    try:
+                        self.sock.server_hostname = self.host
+                    except AttributeError:
+                        pass
 
 
 def _build_connection(connection, source_address, timeout, context=None):
@@ -607,7 +611,8 @@ def build_user_agent():
 
     ua_tuple = (
         'Mozilla/5.0',
-        '(%s; U; %s; en-us)' % (platform.system(), platform.architecture()[0]),
+        '(%s; U; %s; en-us)' % (platform.platform(),
+                                platform.architecture()[0]),
         'Python/%s' % platform.python_version(),
         '(KHTML, like Gecko)',
         'speedtest-cli/%s' % __version__
@@ -666,6 +671,8 @@ def catch_request(request, opener=None):
 
     try:
         uh = _open(request)
+        if request.get_full_url() != uh.geturl():
+            printer('Redirected to %s' % uh.geturl(), debug=True)
         return uh, False
     except HTTP_ERRORS:
         e = get_exception()
@@ -1041,10 +1048,7 @@ class Speedtest(object):
     @property
     def best(self):
         if not self._best:
-            raise SpeedtestMissingBestServer(
-                'get_best_server not called or not able to determine best '
-                'server'
-            )
+            self.get_best_server()
         return self._best
 
     def get_config(self):
@@ -1439,8 +1443,12 @@ class Speedtest(object):
         printer('Best Server:\n%r' % best, debug=True)
         return best
 
-    def download(self, callback=do_nothing):
-        """Test download speed against speedtest.net"""
+    def download(self, callback=do_nothing, threads=None):
+        """Test download speed against speedtest.net
+
+        A ``threads`` value of ``None`` will fall back to those dictated
+        by the speedtest.net configuration
+        """
 
         urls = []
         for size in self.config['sizes']['download']:
@@ -1479,7 +1487,7 @@ class Speedtest(object):
                 finished.append(sum(thread.result))
                 callback(thread.i, request_count, end=True)
 
-        q = Queue(self.config['threads']['download'])
+        q = Queue(threads or self.config['threads']['download'])
         prod_thread = threading.Thread(target=producer,
                                        args=(q, requests, request_count))
         cons_thread = threading.Thread(target=consumer,
@@ -1501,8 +1509,12 @@ class Speedtest(object):
             self.config['threads']['upload'] = 8
         return self.results.download
 
-    def upload(self, callback=do_nothing, pre_allocate=True):
-        """Test upload speed against speedtest.net"""
+    def upload(self, callback=do_nothing, pre_allocate=True, threads=None):
+        """Test upload speed against speedtest.net
+
+        A ``threads`` value of ``None`` will fall back to those dictated
+        by the speedtest.net configuration
+        """
 
         sizes = []
 
@@ -1525,9 +1537,12 @@ class Speedtest(object):
             )
             if pre_allocate:
                 data.pre_allocate()
+
+            headers = {'Content-length': size}
             requests.append(
                 (
-                    build_request(self.best['url'], data, secure=self._secure),
+                    build_request(self.best['url'], data, secure=self._secure,
+                                  headers=headers),
                     size
                 )
             )
@@ -1557,7 +1572,7 @@ class Speedtest(object):
                 finished.append(thread.result)
                 callback(thread.i, request_count, end=True)
 
-        q = Queue(self.config['threads']['upload'])
+        q = Queue(threads or self.config['threads']['upload'])
         prod_thread = threading.Thread(target=producer,
                                        args=(q, requests, request_count))
         cons_thread = threading.Thread(target=consumer,
@@ -1592,7 +1607,8 @@ def ctrl_c(shutdown_event):
 def version():
     """Print the version"""
 
-    printer(__version__)
+    printer('speedtest-cli %s' % __version__)
+    printer('Python %s' % sys.version.replace('\n', ''))
     sys.exit(0)
 
 
@@ -1625,6 +1641,10 @@ def parse_args():
     parser.add_argument('--no-upload', dest='upload', default=True,
                         action='store_const', const=False,
                         help='Do not perform upload test')
+    parser.add_argument('--single', default=False, action='store_true',
+                        help='Only use a single connection instead of '
+                             'multiple. This simulates a typical file '
+                             'transfer.')
     parser.add_argument('--bytes', dest='units', action='store_const',
                         const=('byte', 8), default=('bit', 1),
                         help='Display values in bytes instead of bits. Does '
@@ -1839,7 +1859,10 @@ def shell():
     if args.download:
         printer('Testing download speed', quiet,
                 end=('', '\n')[bool(debug)])
-        speedtest.download(callback=callback)
+        speedtest.download(
+            callback=callback,
+            threads=(None, 1)[args.single]
+        )
         printer('Download: %0.2f M%s/s' %
                 ((results.download / 1000.0 / 1000.0) / args.units[1],
                  args.units[0]),
@@ -1850,7 +1873,11 @@ def shell():
     if args.upload:
         printer('Testing upload speed', quiet,
                 end=('', '\n')[bool(debug)])
-        speedtest.upload(callback=callback, pre_allocate=args.pre_allocate)
+        speedtest.upload(
+            callback=callback,
+            pre_allocate=args.pre_allocate,
+            threads=(None, 1)[args.single]
+        )
         printer('Upload: %0.2f M%s/s' %
                 ((results.upload / 1000.0 / 1000.0) / args.units[1],
                  args.units[0]),
@@ -1888,7 +1915,10 @@ def main():
         e = get_exception()
         # Ignore a successful exit, or argparse exit
         if getattr(e, 'code', 1) not in (0, 2):
-            raise SystemExit('ERROR: %s' % e)
+            msg = '%s' % e
+            if not msg:
+                msg = '%r' % e
+            raise SystemExit('ERROR: %s' % msg)
 
 
 if __name__ == '__main__':
